CREATELIB "utillib"

; Softbite has written two procs (BEGIN_SHADOW) and (END_SHADOW) to help
; you:
;      1) Understand the FORM LOCK issue in intricate detail
;      2) Understand the SHADOW TABLES work-around
;      3) Give you real code to help you "beat the FORM LOCK"
;
; Typically, these routines should be used in place of the PICKFORM
; and FORMKEY operations that are normally performed in multi-table
; editing routines.  The call to the PROCS usually looks something
; like this:
;
;    view the shadow table
;    coedit the master table
;    WAIT table on the master, allowing the user to pick which record to edit
;    LOCK of the master record, with error checking
;
; ** BEGIN_SHADOW("master","1") ; the master/shadow names and the form
;                               ; number is passed to the routine
;
;    moveto the detail table, enbedded into the form
;    WAIT TABLE on the detail records
;    make sure all details are posted
;
; ** END_SHADOW("master") ; master/shadow table name is passed
;
;    unlock the master record
;    loop back to the first WAIT TABLE on the master (user selects another rec)
;
;------------------------------------------------------------------------------
;
; BEGIN_SHADOW
; ++++++++++++
; This proc is called when the master record has been successfully locked
; by you, and you would normally do a PICKFORM on the master in order to
; edit the detail records.
;
; The PROC grabs a copy of the master record and puts it into an array.
; It is essential that YOU lock the master record BEFORE calling this
; routine.  Once this PROC starts, it doesn't touch the master record.
;
; The PROC moves to the master's shadow, then does a PICKFORM to bring up
; the detail records.  A FORM LOCK is not applied, as the shadow is an
; un-keyed replica of the master table and its form.
;
; Finally, the PROC puts the record into a new "row" of the shadow table, and
; issues a RESYNCKEY.  This RESYNCKEY does two things:
;
;       1) Locks the record, so that is really in the shadow table
;       2) Brings the related detail records into the view
;

PROC begin_shadow(table_name,form_number)
   PRIVATE rec_contents
   
   CopyToArray rec_contents
   MOVETO "SHADOW\\"+table_name
   PICKFORM form_number
   Ins
   CopyFromArray rec_contents
   resynckey
   
ENDPROC; BEGIN_SHADOW
WRITELIB "utillib"  begin_shadow
RELEASE PROCS ALL

;
;------------------------------------------------------------------------------
;
; END_SHADOW
; ++++++++++
; Basically, this PROC is called when the editing of the detail records has
; been completed, and all details are posted to the table.  It is a good
; practice to make sure that the detail record is not locked, as this PROC
; forces the unlocking of the last record if it locked (regardless of
; whether the record is a KEYVIOL).
;
; This proc should be called when the cursor is in the DETAIL table!!!
;
; The PROC moves up to the shadow table, and copies the contents of the
; record into an array (in order to put it back into the "real" master
; table.  Typically, the user has not been allowed to change this
; "master" record while using the shadow table, but it is possible that
; the calling script could be written such that changes are allowed.
;
; VERY IMPORTANT:  The user must NOT be allowed to change the field(s)
; that make up the key of the MASTER table!  Place the field(s) as
; DisplayOnly on the form, or make the Master form DisplayOnly through
; the menu choices {Multi} {Table} {DisplayOnly} {Master}.
;
; The PROC deletes the record from the shadow table.  This is necessary
; to keep the shadow table from growing to an unlimited size over time.
;
; The PROC then issues a lock, then unlock operation, as this is the ONLY
; way we know of to be able to toggle out of formview when the LINK LOCK
; is in place.  Remember, we are avoiding the FORM LOCK.  The LINK LOCK
; will still be there.
;
; Finally, the PROC toggles back to table view, moves back to the "real"
; master record, and puts any changes into that record.  A FIRSTSHOW is
; issued to make the shadow table disappear from the screen.
;
; The master record is not unlocked - that is up to you to do in the
; calling script.


PROC end_shadow(table_name)
   PRIVATE rec_contents
   
   LockRecord                    ; force the writing of the detail record,
   UnLockRecord                  ; so that nothing is locked
   
   MOVETO "SHADOW\\"+table_name  ; move up to the master shadow
   CopyToArray rec_contents      ; copy the shadow record into an array
   DEL                           ; delete this shadow record
   
   LockRecord                    ; the lock, then unlock is the only way to
   UnLockRecord                  ; leave the form if the shadow is empty
   
   FormKey                       ; return to form view
   MOVETO table_name             ; move to the real master table
   CopyFromArray rec_contents    ; copy the shadow record into the "real" table
   FirstShow                     ; "hide" the shadow table
   
ENDPROC; END_SHADOW
WRITELIB "utillib"  end_shadow
RELEASE PROCS ALL

INFOLIB "utillib"

