/* listdb2.c

   This code example lists the contents of a table.  The table name can
   be include along with the program name.  If it is not, the program
   prompts for the name of a table.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "pxengine.h"

#define NET_NAME_PATH              "c:\\apps\\paradox3"
#define NET_TYPE                   NOTONNET
#define USER_NAME                  "demoprog"

#define SUCCESS                    0

#define MAX_FIELDS                 255
#define MAX_BUFFER_SIZE            26
#define MAX_FILENAME_SIZE          40

int main(int, char **);
void Intro();
int Initialize(void);
void OpenTable(char *);
void GetTableStructure(void);
void PrintTableHeader(void);
void PrintTable(void);
void PrintTableRecord(void);
void CloseTable(void);
void Terminate(void);

TABLEHANDLE tableHandle;
RECORDHANDLE recHandle;
char fieldNames[MAX_FIELDS][MAX_BUFFER_SIZE];
char fieldTypes[MAX_FIELDS][MAX_BUFFER_SIZE];
int  fieldSize[MAX_FIELDS];
int nFields;
RECORDNUMBER nRecs;

int main(int argc, char **argv)
{
    char filename[MAX_FILENAME_SIZE + 1];
    int pxErr=0,i;

    system("cls");                               /* clear the screen          */
    if (Initialize() == SUCCESS)                 /* initialize system         */
    {
        if (argc > 1)                            /* if table name included at */
            OpenTable(argv[1]);                  /* startup open the table    */
        else
        {
            Intro();                             /* display introduction text */
            printf("Enter table name: ");        /* prompt for table name     */
            scanf("%s",filename);
            OpenTable(filename);                 /* attemp to open table      */
        }
        GetTableStructure();                     /* determines table structure*/
        PrintTableHeader();                      /* prints column headings    */
        PrintTable();                            /* prints records            */
        CloseTable();                            /* closes table              */
        Terminate();                             /* terminates the engine     */
    }
    else
        printf("%s\n", PXErrMsg(pxErr));         /* print error message if    */
                                                 /* the Engine could not be   */
                                                 /* initialized               */

    return(pxErr);                               /* exit and return value of  */
                                                 /* Engine error code         */
}

int Initialize(void)
{
    return(PXNetInit(NET_NAME_PATH, NET_TYPE, USER_NAME));
}

void Intro()
{
    printf("\nThis is a program that reads from a Paradox table,\n"
           "and prints the data to the screen, each record on a \n"
           "separate line.\n\n");
}


void OpenTable(char *tableName)
{
    int pxErr;

    if (PXTblOpen(tableName, &tableHandle, 0, 0))
    {
        printf("Cannot open table %s\n", tableName);
        exit(1);
    }
    if (pxErr = PXRecBufOpen(tableHandle, &recHandle))
        PXErrMsg(pxErr);
}

void GetTableStructure(void)
{
    int i, pxErr;

    if (pxErr = PXTblNRecs(tableHandle, &nRecs))
        PXErrMsg(pxErr);
    if (pxErr = PXRecNFlds(tableHandle, &nFields))
        PXErrMsg(pxErr);
    for(i = 0; i < nFields; i++)
    {
        PXFldName(tableHandle, (FIELDHANDLE) i + 1,  /* field names   */
                  MAX_BUFFER_SIZE, fieldNames[i]);
        PXFldType(tableHandle, (FIELDHANDLE) i + 1,  /* field types   */
                  MAX_BUFFER_SIZE, fieldTypes[i]);
        switch(fieldTypes[i][0])
        {                                /* determine the maximum     */
            case 'N' :                   /* width a field needs to be */
            case '$' :                   /* on ouput by comparing the */
            case 'D' :                   /* data type to field name   */
                fieldSize[i] =  (int) max(10, strlen(fieldNames[i]));
                break;
            case 'S' :
                fieldSize[i] = (int) max(5, strlen(fieldNames[i]));
                break;
           case 'A' :
                fieldSize[i] = (int) max(atoi(&fieldTypes[i][1]),
                strlen(fieldNames[i]));
                break;
        }
    }
}

void PrintTableHeader(void)
{
    int i;                                        /* print the column names   */

    for(i = 0; i < nFields; i++)
        printf("%-*s  ", fieldSize[i], fieldNames[i]);
    putchar('\n');
}

void PrintTable(void)
{
    int i, pxErr;                                /* print the data for each   */
                                                 /* record                    */
    for(i = 0; i < nRecs; i++) 
    {
        if (pxErr = PXRecGoto(tableHandle, i + 1))
            PXErrMsg(pxErr);
        if (pxErr = PXRecGet(tableHandle, recHandle))
            PXErrMsg(pxErr);
        PrintTableRecord();
    }
}

void PrintTableRecord(void)
{
    double dvalue;
    char avalue[BUFSIZ];
    int mo, da, yr, i, pxErr;
    long lvalue;
    short svalue;

    for(i = 0; i < nFields; i++) 
    {
        switch(fieldTypes[i][0]) 
        {
            case 'N':
            case '$':
                if (pxErr = PXGetDoub(recHandle, i + 1, &dvalue))
                    PXErrMsg(pxErr);
                printf("%*g  ", fieldSize[i], dvalue);
                break;
            case 'A':
                if (pxErr = PXGetAlpha(recHandle, i + 1, BUFSIZ, avalue))
                    PXErrMsg(pxErr);
                printf("%-*s  ", fieldSize[i], avalue);
                break;
            case 'D':
                if ( pxErr = PXGetDate(recHandle, i + 1, &lvalue))
                    PXErrMsg(pxErr);
                if ( pxErr = PXDateDecode(lvalue, &mo, &da, &yr))
                    PXErrMsg(pxErr);
                printf("%-02d/%-02d/%-04d  ", mo, da, yr);
                break;
            case 'S':
                if ( pxErr = PXGetShort(recHandle, i + 1, &svalue))
                PXErrMsg(pxErr);
                printf("%*d  ", fieldSize[i], svalue);
                break;
            default:
                printf("unknown\t");
                break;
        }
    }
    putchar('\n');
}

void CloseTable(void)
{
    int pxErr;

    if (pxErr = PXRecBufClose(recHandle))
        PXErrMsg(pxErr);
    if (pxErr = PXTblClose(tableHandle))
        PXErrMsg(pxErr);
}

void Terminate(void)
{
    int pxErr;

    if (pxErr = PXExit())
        PXErrMsg(pxErr);
}
